@@ -270,8 +270,7 @@ exec_data_new (void)
 }
 
 static ExecData *
-exec_data_clone (ExecData *data,
-                 gboolean preserve_argv)
+exec_data_clone (ExecData *data)
 {
   if (data == nullptr)
     return nullptr;
@@ -281,8 +280,7 @@ exec_data_clone (ExecData *data,
   clone->cwd = g_strdup (data->cwd);
 
   /* If FDs were passed, cannot repeat argv. Return data only for env and cwd */
-  if (!preserve_argv ||
-      data->fd_list != nullptr) {
+  if (data->fd_list != nullptr) {
     clone->as_shell = TRUE;
     return clone;
   }
@@ -833,7 +831,6 @@ terminal_screen_reexec_from_screen (TerminalScreen *screen,
 
   g_return_val_if_fail (TERMINAL_IS_SCREEN (parent_screen), FALSE);
 
-  terminal_unref_exec_data ExecData* data = exec_data_clone (parent_screen->priv->exec_data, FALSE);
   gs_free char* cwd = terminal_screen_get_current_dir (parent_screen);
 
   _terminal_debug_print (TERMINAL_DEBUG_PROCESSES,
@@ -843,7 +840,7 @@ terminal_screen_reexec_from_screen (TerminalScreen *screen,
                          cwd);
 
   return terminal_screen_reexec_from_exec_data (screen,
-                                                data,
+                                                nullptr /* exec data */,
                                                 nullptr /* envv */,
                                                 cwd,
                                                 cancellable,
@@ -1584,7 +1581,7 @@ spawn_result_cb (VteTerminal *terminal,
   }
 
   /* Retain info for reexec, if possible */
-  ExecData *new_exec_data = exec_data_clone (exec_data, TRUE);
+  ExecData *new_exec_data = exec_data_clone (exec_data);
   terminal_screen_clear_exec_data (screen, FALSE);
   priv->exec_data = new_exec_data;
   }
